﻿// Funkcje i operatory

module Demo1

open System

/// A very simple constant integer
let int1 = 1

/// A second very simple constant integer
let int2 = 2

/// Add two integers
let int3 = int1 + int2

// Functions on integers  
// ---------------------------------------------------------------

/// A function on integers
let f x = 2*x*x - 5*x + 3

/// The result of a simple computation 
let result = f (int3 + 4)

/// Another function on integers
let increment = fun x -> x + 1

/// Basic function application
let apply (transform : int -> int ) y = transform y

let c101 = apply increment 100

/// Compute the factorial of an integer
let rec factorial n = if n = 0 then 1 else n * factorial (n-1)

/// Compute the highest-common-factor of two integers
let rec hcf a b =                       // notice: 2 parameters separated by spaces
    if a=0 then b
    elif a<b then hcf a (b-a)           // notice: 2 arguments separated by spaces
    else hcf (a-b) b
    // note: function arguments are usually space separated
    // note: 'let rec' defines a recursive function

// Booleans
// ---------------------------------------------------------------

/// A simple boolean value
let boolean1 = true

/// A second simple boolean value
let boolean2 = false

/// Compute a new boolean using ands-and-ors
let boolean3 = not boolean1 && (boolean2 || false)

// Strings
// ---------------------------------------------------------------

/// A simple string
let stringA  = "Hello"

/// A second simple string
let stringB  = "world"

/// "Hello world" computed using string concatentation
let stringC  = stringA + " " + stringB

/// "Hello world" computed using a .NET library function
let stringD = String.Join(" ",[| stringA; stringB |])
  // Try re-typing the above line to see intellisense in action
  // Note, ctrl-J on (partial) identifiers re-activates it
  
// Operations on functions
// ---------------------------------------------------------------

let function1 x = x + 1
let function2 x = x * 2
let h = function1 >> function2
let result5 = h 100

let g = function1 << function2
let result6 = g 100

///Pipelining example
let result7 = result6 |> function1 |> function2

let result8 = (function2 <| result6) |> function1

// Operators
// ---------------------------------------------------------------
let r9 = 2 + 2
let r10 = 3.5 + 2.5
let r11 = "Hello " + "world"

///Defined operator
let inline (+?) (x: int) (y: int) = x + 2*y
printf "%d" (10 +? 1)